//
//  KMGimbalManager.h
//  Koupon
//
//  Created by Brian Reinhart on 12/31/13.
//
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <FYX/FYX.h>
#import <FYX/FYXSightingManager.h>
#import <FYX/FYXVisitManager.h>
#import <FYX/FYXTransmitter.h>
#import <ContextLocation/QLContextPlaceConnector.h>
#import <ContextCore/QLContextCoreConnector.h>
#import <FYX/FYXLogging.h>
#import "Koupon.h"
#import "KMProximityEvent.h"
#import "KMGeofenceEvent.h"

extern NSString *const KMGimbalManagerBeaconServiceStartedNotification;
extern NSString *const KMGimbalManagerBeaconServiceFailedToStartNotification;
extern NSString *const KMGimbalManagerReceivedSightingNotification;
extern NSString *const KMGimbalManagerReceivedArrivalNotification;
extern NSString *const KMGimbalManagerReceivedDepartureNotification;
extern NSString *const KMGimbalManagerDidEnterGeofenceNotification;
extern NSString *const KMGimbalManagerDidExitGeofenceNotification;

@protocol KouponGimbalManagerDelegate <NSObject>
@optional
/** Called on a successful initialization of the beacon service.
  */
- (void) beaconServiceStarted;
/** Called on an unsuccessful initialization attempt.
 @param error NSError
*/
- (void) beaconServiceFailedToStart: (NSError *) error;
/** Called for each "heartbeat" received by a provisioned beacon.
 @see KMProximityEvent class
*/
- (void) receivedSighting: (KMProximityEvent*) proximityEvent;
/** Called for each arrival event (usually the first heartbeat)
 @see KMProximityEvent class
 */
- (void) receivedArrival: (KMProximityEvent*) proximityEvent;
/** Called for each departure event.
 The departure event occurs after 'x' number of seconds after the last heartbeat was obtained.  Where 'x' is the value of "VisitDepartureInterval" in the KouponConfiguration.plist
 */
- (void) receivedDeparture: (KMProximityEvent*) proximityEvent;
/** Called when the device enters a geofence.
 @see KMGeofenceEvent class
 */
- (void) didEnterGeofence: (KMGeofenceEvent*) geofenceEvent;
/** Called when the device leaves a geofence
 @see KMGeofenceEvent class
 */
- (void) didExitGeofence: (KMGeofenceEvent*) geofenceEvent;
@end


/** This class manages the bridge between the Gimbal APIs and the Koupon Media APIs.  
 The class is a singleton that should always be referenced through the static selector [KouponGimbalManager sharedManager].
 */


@interface KouponGimbalManager : NSObject <FYXServiceDelegate, QLContextPlaceConnectorDelegate, FYXVisitDelegate> {
    NSMutableDictionary *offerDict;
    BOOL scanForBeacons;
    NSDate *geoEnterTime;
    NSDateFormatter * dateFormatter;
    NSArray *signalSmoothingArray;
    NSMutableDictionary *gimbalConfig;
    Koupon *kouponSDK;
    QLContextCoreConnector *connector;
}

@property (nonatomic, weak) id <KouponGimbalManagerDelegate> delegate;
@property (nonatomic) QLContextPlaceConnector *placeConnector;
@property (nonatomic) FYXVisitManager *visitManager;
@property (nonatomic) NSMutableDictionary *visitHistoryDictionary;
@property (nonatomic, readonly) BOOL BeaconsEnabled;
@property (nonatomic, readonly) BOOL GeofenceEnabled;

/** Singleton accessor.  Use this static method for getting the shared instance of the KouponGimbalManager class.
 @return KouponGimbalManager
 */
+ (KouponGimbalManager *)sharedManager;
/** Initializes the GimbalServices with a specified delegate and Koupon SDK instance.
 
 @param delegate id <KouponGimbalManagerDelegate>
 @param sdk Koupon
 */
- (void) initGimbalServicesWithDelegate: (id <KouponGimbalManagerDelegate>) delegate KouponSDK: (Koupon*) sdk;
/** Tells the KouponGimbalManager to begin listening for beacon events.
 */
- (void) startListeningForVisits;
/** Tells the KouponGimbalManager to stop listening for beacon events.
 */
- (void) stopListeningForVisits;

- (void) showPrivacyPanelOnViewController: (UIViewController *) viewController;

@end



